/* $Id: api.c 6 2023-04-24 08:55:48Z nishi $ */
#include <swcli/api.h>

#include <curl/curl.h>
#include <cjson/cJSON.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

extern int argc;
extern char** argv;

char* swcli_api_buffer;
size_t swcli_api_buffer_length;

size_t swcli_api_write(void* ptr, size_t size, size_t nmemb, void* stream){
	size_t old_length = swcli_api_buffer_length;
	swcli_api_buffer_length += size * nmemb;
	swcli_api_buffer = realloc(swcli_api_buffer, swcli_api_buffer_length + 1);
	swcli_api_buffer[swcli_api_buffer_length] = 0;
	memcpy(swcli_api_buffer + old_length, ptr, size * nmemb);
	return size * nmemb;
}

cJSON* swcli_api(const char* url, const char* data){
	cJSON* result = NULL;
	swcli_api_buffer = malloc(1);
	swcli_api_buffer[0] = 0;
	swcli_api_buffer_length = 0;
	CURL* curl;
	curl_global_init(CURL_GLOBAL_ALL);
	curl = curl_easy_init();
	curl_easy_setopt(curl, CURLOPT_URL, url);
	curl_easy_setopt(curl, CURLOPT_VERBOSE, 0L);
	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1L);
	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, swcli_api_write);
	if(data != NULL) curl_easy_setopt(curl, CURLOPT_POSTFIELDS, data);
	CURLcode res = curl_easy_perform(curl);
	if(res != CURLE_OK){
		fprintf(stderr, "%s: %s\n", argv[0], curl_easy_strerror(res));
	}else{
		result = cJSON_Parse(swcli_api_buffer);
		if(result == NULL){
			fprintf(stderr, "%s: Parse error\n", argv[0]);
		}else{
			if(cJSON_GetObjectItem(result, "code")->valuedouble != 0){
				fprintf(stderr, "%s: API error: %s\n", argv[0], cJSON_GetObjectItem(result, "error")->valuestring);
				cJSON_Delete(result);
				result = NULL;
			}
		}
	}
	curl_easy_cleanup(curl);
	curl_global_cleanup();
	free(swcli_api_buffer);
	return result;
}
